
--  Copyright (C) 2002, 2003, 2004, 2005, 2006 Balint Kis (balint@k-i-s.net)

--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 2 of the License, or
--  (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

----------------------------------------------------------
-- v2.5
----------------------------------------------------------

create table version (
  identifier            integer not null primary key,
  description           varchar(300),
  major                 smallint,
  minor                 smallint,
  variation             varchar(1)
);

----------------------------------------------------------
-- general tables used mostly to replace strings in the db
----------------------------------------------------------
create sequence country_id_sequence start 1;
create table country (
  identifier            integer not null primary key,
  code                  varchar(5),
  value                 varchar(40)
);

create sequence state_id_sequence start 1;
create table state (
  identifier            integer not null primary key,
  code                  varchar(5),
  value                 varchar(40)
);


create sequence aperture_id_sequence start 1;
create table aperture (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence focal_length_id_sequence start 1;
create table focal_length (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence shutter_id_sequence start 1;
create table shutter (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence camera_metering_id_sequence start 1;
create table camera_metering (
  identifier            integer not null primary key,
  value                 varchar(40)
);

create sequence camera_program_id_sequence start 1;
create table camera_program (
  identifier            integer not null primary key,
  value                 varchar(40)
);

create sequence flash_mode_id_sequence start 1;
create table flash_mode (
  identifier            integer not null primary key,
  value                 varchar(40)
);

create sequence ev_comp_id_sequence start 1;
create table ev_comp (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence scan_resolution_id_sequence start 1;
create table scan_resolution (
  identifier            integer not null primary key,
  value                 varchar(6)
);

create sequence scan_bitdepth_id_sequence start 1;
create table scan_bitdepth (
  identifier            integer not null primary key,
  value                 varchar(6)
);

create sequence scan_multiscan_id_sequence start 1;
create table scan_multiscan (
  identifier            integer not null primary key,
  value                 integer
);

create sequence access_type_id_sequence start 1;
create table access_type (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence user_type_id_sequence start 1;
create table user_type (
  identifier            integer not null primary key,
  value                 varchar(15)
);

----------------------------------------------------------
-- users and manufacturer tables
----------------------------------------------------------
create sequence contact_info_id_sequence start 2;
create table contact_info (
  identifier            integer not null primary key,
  address1              varchar(100),
  address2              varchar(100),
  city                  varchar(100),
  zipcode               varchar(10),
  state                 integer references state(identifier),
  country               integer references country(identifier),
  phone                 varchar(40),
  email                 varchar(150),
  url                   varchar(150)
);
create index contact_info_idx on contact_info(identifier);

create sequence preferences_id_sequence start 2;
create table preferences (
  identifier            integer not null primary key,
  print_out             char(3) check (print_out in ('ps', 'pdf')),
  start_hour            integer check (start_hour >= 0 and start_hour < 24),
  end_hour              integer check (start_hour >= 0 and start_hour < 24),
  start_day             integer check (start_day in (1, 0)),
  num_of_rows           integer,
  num_of_columns        integer,
  copyright_statement   text,
  folder_view           char(5) check (folder_view in ('slide', 'list')),
  num_of_rows_l         integer,
  sharpen_image_radius  float,
  sharpen_image_sigma   float,
  sharpen_thumb_radius  float,
  sharpen_thumb_sigma   float
);

create sequence users_id_sequence start 2;
create table users (
  identifier            integer not null primary key,
  first_name            varchar(50),
  last_name             varchar(50),
  company               varchar(50),
  username              varchar(30),
  password              varchar(30),
  session_id            varchar(32),
  contact               integer references contact_info(identifier),
  member_since          timestamp,
  preferences           integer references preferences(identifier),
  type                  integer not null references user_type(identifier)
);
create index users_idx on users(identifier);

create sequence manufacturer_id_sequence start 1;
create table manufacturer (
  identifier            integer not null primary key,
  name                  varchar(100),
  url                   varchar(100),
  last_modified_date 	timestamp,
  last_modifying_users  integer references users(identifier)
);

create sequence location_id_sequence start 1;
create table location ( -- location is the place the photo was taken
  identifier            integer not null primary key,
  place                 varchar(300),
  state                 integer references state(identifier),
  country               integer references country(identifier),
  last_modified_date 	timestamp,
  last_modifying_users  integer references users(identifier),
  city                  varchar(300),
  latitude              varchar(10),
  longitude             varchar(10)
);

----------------------------------------------------------
-- equipment related tables
----------------------------------------------------------
create sequence film_iso_id_sequence start 1;
create table film_iso (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence film_format_id_sequence start 1;
create table film_format (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence film_type_id_sequence start 1;
create table film_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  iso                   integer not null references film_iso(identifier),
  format                integer not null references film_format(identifier),
  last_modified_date 	timestamp,
  last_modifying_users	integer references users(identifier)
);
create index film_type_manufacturer_idx on film_type(manufacturer);

create sequence film_id_sequence start 1;
create table film (
  identifier            integer not null primary key,
  type                  integer not null references film_type(identifier),
  users                 integer not null references users(identifier),
  access_rights         integer not null references access_type(identifier)
);
create index film_type_idx  on film(type);
create index film_users_idx on film(users);

create sequence lens_type_id_sequence start 1;
create table lens_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  min_focal_length      integer not null references focal_length(identifier),
  max_focal_length      integer not null references focal_length(identifier),
  min_aperture          integer not null references aperture(identifier),
  max_aperture          integer not null references aperture(identifier),
  last_modified_date 	timestamp,
  last_modifying_users	integer references users(identifier)
);
create index lens_type_manufacturer_idx on lens_type(manufacturer);

create sequence lens_id_sequence start 1;
create table lens (
  identifier            integer not null primary key,
  type                  integer not null references lens_type(identifier),
  users                 integer not null references users(identifier),
  serial_number         varchar(20),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index lens_type_idx  on lens(type);
create index lens_users_idx on lens(users);

create sequence camera_type_id_sequence start 1;
create table camera_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  last_modified_date 	timestamp,
  last_modifying_users	integer references users(identifier)
);
create index camera_type_manufacturer_idx on camera_type(manufacturer);

create sequence camera_id_sequence start 1;
create table camera (
  identifier            integer not null primary key,
  type                  integer references camera_type(identifier),
  users                 integer references users(identifier),
  serial_number         varchar(20),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index camera_type_idx  on camera(type);
create index camera_users_idx on camera(users);

create sequence flash_type_id_sequence start 1;
create table flash_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  last_modified_date 	timestamp,
  last_modifying_users	integer references users(identifier)
);
create index flash_type_manufacturer_idx on flash_type(manufacturer);

create sequence flash_id_sequence start 1;
create table flash (
  identifier            integer not null primary key,
  type                  integer not null references flash_type(identifier),
  users                 integer not null references users(identifier),
  serial_number         varchar(20),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index flash_type_idx  on flash(type);
create index flash_users_idx on flash(users);

create sequence scanner_type_id_sequence start 1;
create table scanner_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  last_modified_date    timestamp,
  last_modifying_users  integer references users(identifier)
);
create index scanner_type_manufacturer_idx on scanner_type(manufacturer);

create sequence scanner_id_sequence start 1;
create table scanner (
  identifier            integer not null primary key,
  type                  integer not null references scanner_type(identifier),
  users                 integer not null references users(identifier),
  serial_number         varchar(20),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index scanner_type_idx  on scanner(type);
create index scanner_users_idx on scanner(users);

create sequence support_type_id_sequence start 1;
create table support_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  last_modified_date    timestamp,
  last_modifying_users  integer references users(identifier)
);
create index support_type_manufacturer_idx on support_type(manufacturer);

create sequence support_id_sequence start 1;
create table support (
  identifier            integer not null primary key,
  type                  integer not null references support_type(identifier),
  users                 integer not null references users(identifier),
  serial_number         varchar(20),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index support_type_idx  on support(type);
create index support_users_idx on support(users);

create sequence filter_type_id_sequence start 1;
create table filter_type (
  identifier            integer not null primary key,
  manufacturer          integer not null references manufacturer(identifier),
  model                 varchar(100),
  variation             varchar(100),
  last_modified_date 	timestamp,
  last_modifying_users	integer references users(identifier)
);
create index filter_type_manufacturer_idx on filter_type(manufacturer);

create sequence filter_id_sequence start 1;
create table filter (
  identifier            integer not null primary key,
  type                  integer not null references filter_type(identifier),
  users                 integer not null references users(identifier),
  date_of_purchase      timestamp,
  purchased_new         char(1) check (purchased_new in ('t', 'f')),
  access_rights         integer not null references access_type(identifier)
);
create index filter_type_idx  on filter(type);
create index filter_users_idx on filter(users);

create sequence folder_id_sequence start 2;
create table folder (
  identifier            integer not null primary key,
  parent_folder         integer references folder(identifier),
  users                 integer not null references users(identifier),       -- the users who uploaded the picture, not necessarily the owner
  caption               text,
  date_of_creation      timestamp,
  access_rights         integer not null references access_type(identifier)
);
create index folder_id_idx    on folder(identifier);
create index folder_users_idx on folder(users);

create sequence album_id_sequence start 1;
create table album (
  identifier            integer not null primary key,
  users                 integer not null references users(identifier),       -- the users who uploaded the picture, not necessarily the owner
  caption               text,
  date_of_creation      timestamp,
  access_rights         integer not null references access_type(identifier)
);
create index album_id_idx    on album(identifier);
create index album_users_idx on album(users);

----------------------------------------------------------
-- photo tables
----------------------------------------------------------
create sequence photo_id_sequence start 1;
create table photo (
  identifier            integer not null primary key,
  users                 integer not null references users(identifier),       -- the users wh uploaded the picture, not necessarily the owner
  folder                integer not null references folder(identifier),
  location              integer references location(identifier),
  caption               text,             -- a novel describing how the picture was made, or what can we see on it
  date_of_exposure      timestamp,        -- date when the picture was taken
  small_image_path      varchar(500),     -- path to the thumbnail
  medium_image_path     varchar(500),     -- path to the medium image
  large_image_path      varchar(500),     -- path to the highres image
  access_rights         integer not null references access_type(identifier)
);

create index photo_id_idx       on photo(identifier);
create index photo_users_idx    on photo(users);
create index photo_location_idx on photo(location);

create sequence photo_dupe_id_sequence start 1;
create table photo_dupe (
  identifier            integer not null primary key,
  key                   integer not null,
  photo                 integer not null references photo(identifier),
  date_of_creation      timestamp,        -- date when it was inserted in the system
  storage_location      varchar(50),
  storage_index         varchar(10),
  remark                text              -- dupe remark
);

create sequence photo_tech_id_sequence start 1;
create table photo_tech (
  identifier            integer not null primary key,
  photo                 integer not null references photo(identifier),
  camera                integer references camera(identifier),
  lens                  integer references lens(identifier),
  film                  integer references film(identifier),
  flash                 integer references flash(identifier),
  scanner               integer references scanner(identifier),
  filter                integer references filter(identifier),
  scan_resolution       integer references scan_resolution(identifier),
  aperture              integer references aperture(identifier),        -- key from aperture, must be within the lens limits
  focal_length          integer references focal_length(identifier),    -- key from focal length, must be within the lens limits
  shutter               integer references shutter(identifier),
  camera_metering       integer references camera_metering(identifier),
  camera_program        integer references camera_program(identifier),
  flash_mode            integer references flash_mode(identifier),
  exposure_comp         integer references ev_comp(identifier),
  flash_comp            integer references ev_comp(identifier),
  ev_difference         integer references ev_comp(identifier),
  iso_override          integer references film_iso(identifier),
  support               integer references support(identifier),
  scan_bitdepth         integer references scan_bitdepth(identifier),
  scan_multiscan        integer references scan_multiscan(identifier)
);

create index photo_tech_id_idx              on photo_tech(identifier);
create index photo_tech_photo_idx           on photo_tech(photo);

create sequence rating_id_sequence start 1;
create table rating (
  identifier            integer not null primary key,
  photo                 integer not null references photo(identifier),
  users                 integer not null references users(identifier),
  date_of_creation      timestamp,
  value                 integer not null
);

create index rating_id_idx    on rating(identifier);
create index rating_photo_idx on rating(photo);
create index rating_users_idx on rating(users);

create sequence album_content_id_sequence start 1;
create table album_content (
  identifier            integer not null primary key,
  photo                 integer not null references photo(identifier),
  album                 integer not null references album(identifier)
);

create index album_content_id_idx    on album_content(identifier);
create index album_content_photo_idx on album_content(photo);
create index album_content_album_idx on album_content(album);

create sequence submission_status_id_sequence start 1;
create table submission_status (
  identifier            integer not null primary key,
  value                 varchar(10)
);

create sequence submission_id_sequence start 1;
create table submission (
  identifier            integer not null primary key,
  photo_dupe            integer not null references photo_dupe(identifier),
  status                integer not null references submission_status(identifier),
  users                 integer not null references users(identifier),
  date_of_creation      timestamp,
  remark                text
);

----------------------------------------------------------
-- printer tables
----------------------------------------------------------

create sequence paper_id_sequence start 1;
create table paper ( -- labels, for printing
  identifier            integer not null primary key,
  name                  varchar(300),
  size_x                float,
  size_y                float,
  last_modified_date 	timestamp,
  last_modifying_users  integer references users(identifier)
);
create index paper_idx on paper(identifier);

create sequence label_id_sequence start 1;
create table label ( -- labels, for printing
  identifier            integer not null primary key,
  users                 integer references users(identifier),
  manufacturer          integer references manufacturer(identifier),
  name                  varchar(300),
  space_x               float,
  space_y               float,
  size_x                float,
  size_y                float,
  offset_x              float,
  offset_y              float,
  columns               float,
  rows                  float,
  paper                 integer not null references paper(identifier),
  date_of_creation      timestamp,
  last_modified_date 	timestamp
);
create index label_idx on label(identifier);

create sequence printer_spooler_id_sequence start 1;
create table printer_spooler (
  identifier            integer not null primary key,
  users                 integer references users(identifier),
  photo                 integer references photo(identifier)
);

----------------------------------------------------------
-- calendar tables
----------------------------------------------------------
create sequence client_status_id_sequence start 1;
create table client_status (
  identifier            integer not null primary key,
  value                 varchar(12)
);

create sequence client_id_sequence start 1;
create table client (
  identifier            integer not null primary key,
  users                 integer not null references users(identifier),
  client                integer not null references users(identifier), -- client of user
  date_of_creation      timestamp,
  status                integer not null references client_status(identifier)
);
create index client_id_idx    on client(identifier);

create sequence calendar_id_sequence start 1;
create table calendar (
  identifier            integer not null primary key,
  users                 integer not null references users(identifier),
  start_date            timestamp,
  end_date              timestamp,
  date_of_creation      timestamp,
  remark                text
);
create index calendar_id_idx    on calendar(identifier);

create sequence print_id_sequence start 1;


----------------------------------------------------------
-- functions
----------------------------------------------------------
create function count_users() returns bigint
as 'select count(identifier) from users' language 'sql';

create function count_photos() returns bigint
as 'select count(identifier) from photo' language 'sql';
create function count_photos(integer) returns bigint
as 'select count(identifier) from photo where access_rights <= $1' language 'sql';

create function count_photos_by_user(integer) returns bigint
as 'select count(identifier) from photo where users = $1' language 'sql';
create function count_photos_by_user(integer, integer) returns bigint
as 'select count(photo.identifier)
      from photo, folder
     where photo.folder=folder.identifier
       and photo.users = $1
       and photo.access_rights <= $2
       and folder.access_rights <= $2' language 'sql';

create function count_folders_by_user(integer) returns bigint
as 'select count(identifier) from folder where users = $1' language 'sql';
create function count_folders_by_user(integer, integer) returns bigint
as 'select count(identifier) from folder where users = $1 and access_rights <= $2' language 'sql';

create function count_albums_by_user(integer) returns bigint
as 'select count(identifier) from album where users = $1' language 'sql';
create function count_albums_by_user(integer, integer) returns bigint
as 'select count(identifier) from album where users = $1 and access_rights <= $2' language 'sql';

create function count_photos_by_folder(integer) returns bigint
as 'select count(identifier) from photo where folder = $1' language 'sql';
create function count_photos_by_folder(integer, integer) returns bigint
as 'select count(identifier) from photo where folder = $1 and access_rights <= $2' language 'sql';

create function count_photos_by_album(integer) returns bigint
as 'select count(photo.identifier)
     from album_content, photo
    where album_content.photo=photo.identifier
      and album = $1' language 'sql';

create function count_photos_by_album(integer, integer) returns bigint
as 'select count(photo.identifier)
      from album_content, photo, album
     where album_content.photo=photo.identifier
       and album_content.album=album.identifier
       and album_content.album = $1
       and photo.access_rights <= $2' language 'sql';

create function count_equipment_by_user(integer) returns bigint
as 'select ((select count(identifier) from lens where users = $1) +
            (select count(identifier) from camera where users = $1) +
            (select count(identifier) from flash where users = $1) +
            (select count(identifier) from scanner where users = $1) +
            (select count(identifier) from film where users = $1) +
            (select count(identifier) from support where users = $1) +
            (select count(identifier) from flash where users = $1))' language 'sql';

create function count_equipment_by_user(integer, integer) returns bigint
as 'select ((select count(identifier) from lens where users = $1 and access_rights <= $2) +
            (select count(identifier) from camera where users = $1 and access_rights <= $2) +
            (select count(identifier) from flash where users = $1 and access_rights <= $2) +
            (select count(identifier) from scanner where users = $1 and access_rights <= $2) +
            (select count(identifier) from film where users = $1 and access_rights <= $2) +
            (select count(identifier) from support where users = $1 and access_rights <= $2) +
            (select count(identifier) from flash where users = $1 and access_rights <= $2))' language 'sql';

create function number_of_lens_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, lens
     where photo_tech.lens=lens.identifier
       and lens.identifier = $1' language 'sql';

create function number_of_camera_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, camera
     where photo_tech.camera=camera.identifier
       and camera.identifier = $1' language 'sql';

create function number_of_flash_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, flash
     where photo_tech.flash=flash.identifier
       and flash.identifier = $1' language 'sql';

create function number_of_scanner_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, scanner
     where photo_tech.scanner=scanner.identifier
       and scanner.identifier = $1' language 'sql';

create function number_of_filter_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, filter
     where photo_tech.filter=filter.identifier
       and filter.identifier = $1' language 'sql';

create function number_of_film_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, film
     where photo_tech.film=film.identifier
       and film.identifier = $1' language 'sql';

create function number_of_support_references(integer) returns bigint
as 'select count(photo_tech.identifier)
      from photo_tech, support
     where photo_tech.support=support.identifier
       and support.identifier = $1' language 'sql';

create function number_of_location_references(integer) returns bigint
as 'select count(photo.identifier)
      from photo, location
     where photo.location=location.identifier
       and location.identifier = $1' language 'sql';

create function number_of_manufacturer_ref(integer) returns bigint
as 'select ((select count(identifier) from lens_type where manufacturer = $1) +
            (select count(identifier) from camera_type where manufacturer = $1) +
            (select count(identifier) from flash_type where manufacturer = $1) +
            (select count(identifier) from scanner_type where manufacturer = $1) +
            (select count(identifier) from support_type where manufacturer = $1) +
            (select count(identifier) from film_type where manufacturer = $1) +
            (select count(identifier) from flash_type where manufacturer = $1))' language 'sql';

create function number_of_dupes(integer) returns bigint
as 'select count(identifier) from photo_dupe where photo = $1' language 'sql';

create function is_user_my_client(integer, integer) returns bigint
as 'select count(identifier)
      from client
     where client.users = $1
       and client.client = $2
       and status=2' language 'sql';

----------------------------------------------------------
-- views
----------------------------------------------------------

create view view_submission as
  select submission.identifier, photo_dupe, photo_dupe.photo, photo_dupe.key, users.identifier as uidentifier, first_name, last_name, submission_status.value, submission.date_of_creation, submission.remark
    from submission, users, photo_dupe, submission_status
   where submission.users = users.identifier
     and submission.photo_dupe = photo_dupe.identifier
     and submission.status = submission_status.identifier
   order by submission.date_of_creation, users.last_name, users.first_name;

create view view_contact_info as
  select identifier, address1, address2, city, zipcode,
         (select value from state where state.identifier=contact_info.state) as state,
         (select value from country where country.identifier=contact_info.country) as country,
         phone, email, url
    from contact_info;

create view view_location as
  select location.identifier, location.place, location.city,
         (select value from state where state.identifier=location.state) as state,
         (select value from country where country.identifier=location.country) as country,
         last_modifying_users, last_modified_date,
         latitude, longitude
    from location;

create view view_client as
  select client.identifier, client.users, client.client, users.last_name, users.first_name, date_of_creation, client_status.value, client.status, users.type
    from client, users, client_status
   where client.client = users.identifier
     and client.status = client_status.identifier
   order by users.last_name, users.first_name;

create view view_contact as
  select client.identifier, client.users, client.client, users.last_name, users.first_name, date_of_creation, client_status.value, client.status
    from client, users, client_status
   where client.client = users.identifier
     and client.status = client_status.identifier
   order by users.last_name, users.first_name;

create view view_users_subscribed_to as
  select client.identifier, client.users, client.client, users.last_name, users.first_name, date_of_creation, client_status.value, client.status
    from client, users, client_status
   where client.users = users.identifier
     and client.status = client_status.identifier
   order by users.last_name, users.first_name;

create view view_camera as
  select camera.identifier, manufacturer.name, model, variation, camera.serial_number, camera.users, date_of_purchase, camera.purchased_new, manufacturer.url, (access_type.value) as access_rights
    from camera_type, camera, manufacturer, access_type
   where camera_type.manufacturer = manufacturer.identifier
     and camera_type.identifier = camera.type
     and camera.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

create view view_camera_type as
  select camera_type.identifier, manufacturer.name, model, variation, manufacturer.url
    from camera_type, manufacturer
   where camera_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

create view view_flash as
  select flash.identifier, manufacturer.name, model, variation, flash.serial_number, flash.users, date_of_purchase, flash.purchased_new, manufacturer.url, (access_type.value) as access_rights
    from flash_type, flash, manufacturer, access_type
   where flash_type.manufacturer = manufacturer.identifier
     and flash_type.identifier = flash.type
     and flash.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

create view view_flash_type as
  select flash_type.identifier, manufacturer.name, model, variation, manufacturer.url
    from flash_type, manufacturer
   where flash_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

create view view_scanner as
  select scanner.identifier, manufacturer.name, model, variation, scanner.serial_number, scanner.users, date_of_purchase, scanner.purchased_new, manufacturer.url, (access_type.value) as access_rights
    from scanner_type, scanner, manufacturer, access_type
   where scanner_type.manufacturer = manufacturer.identifier
     and scanner_type.identifier = scanner.type
     and scanner.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

create view view_scanner_type as
  select scanner_type.identifier, manufacturer.name, model, variation, manufacturer.url
    from scanner_type, manufacturer
   where scanner_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

create view view_support as
  select support.identifier, manufacturer.name, model, variation, support.serial_number, support.users, date_of_purchase, support.purchased_new, manufacturer.url, (access_type.value) as access_rights
    from support_type, support, manufacturer, access_type
   where support_type.manufacturer = manufacturer.identifier
     and support_type.identifier = support.type
     and support.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

create view view_support_type as
  select support_type.identifier, manufacturer.name, model, variation, manufacturer.url
    from support_type, manufacturer
   where support_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

create view view_filter as
  select filter.identifier, manufacturer.name, model, variation, users, date_of_purchase, purchased_new, manufacturer.url, (access_type.value) as access_rights
    from filter_type, filter, manufacturer, access_type
   where filter_type.manufacturer = manufacturer.identifier
     and filter_type.identifier = filter.type
     and filter.access_rights = access_type.identifier
   order by manufacturer.name, model, variation;

create view view_filter_type as
  select filter_type.identifier, manufacturer.name, model, variation, manufacturer.url
    from filter_type, manufacturer
   where filter_type.manufacturer = manufacturer.identifier
   order by manufacturer.name, model, variation;

create view view_film as
  select film.identifier, manufacturer.name, model, film_iso.value as iso, film_format.value as format, film.users, manufacturer.url, (access_type.value) as access_rights
    from film_type, film, manufacturer, film_iso, film_format, access_type
   where film_type.manufacturer = manufacturer.identifier
     and film_type.identifier = film.type
     and film_type.iso = film_iso.identifier
     and film_type.format = film_format.identifier
     and film.access_rights = access_type.identifier
   order by manufacturer.name, model, film_iso.value;

create view view_film_type as
  select film_type.identifier, manufacturer.name, model, film_iso.value as iso, film_format.value as format, manufacturer.url
    from film_type, manufacturer, film_iso, film_format
   where film_type.manufacturer = manufacturer.identifier
     and film_type.iso = film_iso.identifier
     and film_type.format = film_format.identifier
   order by manufacturer.name, model, film_iso.value;

create view view_lens as
  select lens.identifier, manufacturer.name, model, variation, s_min_aperture, s_max_aperture, s_min_focal_length, s_max_focal_length, lens.serial_number, lens.users, date_of_purchase, lens.purchased_new, manufacturer.url, (access_type.value) as access_rights
    from lens_type, lens, manufacturer, access_type,
         (select identifier as identifier, value as s_min_focal_length from focal_length) as t_min_focal_length,
         (select identifier as identifier, value as s_max_focal_length from focal_length) as t_max_focal_length,
         (select identifier as identifier, value as s_min_aperture from aperture) as t_min_aperture,
         (select identifier as identifier, value as s_max_aperture from aperture) as t_max_aperture
   where lens_type.manufacturer = manufacturer.identifier
     and lens_type.identifier = lens.type
     and lens_type.min_focal_length = t_min_focal_length.identifier
     and lens_type.max_focal_length = t_max_focal_length.identifier
     and lens_type.min_aperture = t_min_aperture.identifier
     and lens_type.max_aperture = t_max_aperture.identifier
     and lens.access_rights = access_type.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

create view view_lens_type as
  select lens_type.identifier, manufacturer.name, model, variation, s_min_aperture, s_max_aperture, s_min_focal_length, s_max_focal_length, manufacturer.url
    from lens_type, manufacturer,
         (select identifier as identifier, value as s_min_focal_length from focal_length) as t_min_focal_length,
         (select identifier as identifier, value as s_max_focal_length from focal_length) as t_max_focal_length,
         (select identifier as identifier, value as s_min_aperture from aperture) as t_min_aperture,
         (select identifier as identifier, value as s_max_aperture from aperture) as t_max_aperture
   where lens_type.manufacturer = manufacturer.identifier
     and lens_type.min_focal_length = t_min_focal_length.identifier
     and lens_type.max_focal_length = t_max_focal_length.identifier
     and lens_type.min_aperture = t_min_aperture.identifier
     and lens_type.max_aperture = t_max_aperture.identifier
   order by manufacturer.name, s_min_focal_length, model, variation;

create view view_photo as
  select photo.identifier, caption, users, date_of_exposure, date_of_creation,
         storage_location, storage_index, small_image_path, medium_image_path,
         large_image_path, folder, (access_type.value) as access_rights
    from photo, photo_dupe, access_type
   where photo.identifier = photo_dupe.photo
     and photo.access_rights = access_type.identifier
   order by photo.identifier;

create view view_random_photo as
  select small_image_path, medium_image_path, large_image_path
    from photo
   where photo.identifier=(select cast(random()*count_photos() as integer));

create view view_rating as
  select rating.identifier, photo, users, date_of_creation, value, first_name, last_name
    from rating, users
   where rating.users=users.identifier;
